/*
 * @Author       : Eleanor Mao
 * @Date         : 2024-04-09 10:36:44
 * @LastEditTime : 2024-04-09 10:36:44
 *
 *
 */
import React, { ChangeEvent, FC, KeyboardEvent, useEffect, useMemo, useRef, useState } from "react";
import Grid from '@mui/material/Grid';
import Box from '@mui/material/Box';
import Button from '@mui/material/Button';
import TextField from '@mui/material/TextField';
import SendIcon from '@mui/icons-material/Send';
import List from '@mui/material/List';
import Chip from '@mui/material/Chip';
import Divider from '@mui/material/Divider';
import { ChatItem } from "./ChatItem";
import Stack from '@mui/material/Stack';
import { cloneDeep, uniqueId } from 'lodash';
import Typography from "@mui/material/Typography";
import { useLocation, useNavigate, useParams } from "react-router-dom";
import axios from 'axios';
import { Action, Definition, DSL2Json, Json2Preview, Json2Yml } from "./loadYml";
import RefreshIcon from '@mui/icons-material/Refresh';
import IconButton from '@mui/material/IconButton';
import Tooltip from '@mui/material/Tooltip';
import { EmailSender, EmailValue } from "./EmailSender";
import { AIBot } from "./AIBot";
import { GlipSender } from "./GlipSender";
import { ScriptBlock } from "./Script";
import { message } from 'mui-message';
import { isChatBot } from "./transform";

const DefaultMessage = [
  'Hello! Please tell me what kind of workflow you want to generate!\n',
  'Currently the node types I support are: Chat, BotMessage, GlipSender, Script'].join('\n');

const DefaultMessageZh = [
  '你好(*´▽｀)ノノ\n 快告诉我你想生成什么样的工作流吧 ❗',
].join('\n');

const DefaultEditMessageZh = [
  '又见面啦(*╹▽╹*)\n 你已经生成过工作流啦，如果有需要修改的请告诉我吧 ❗',
].join('\n');

function getDefaultMessage(edit?: boolean): string {
  if (edit) {
    return DefaultEditMessageZh;
  }

  return DefaultMessageZh;
}

type PreValue = string | undefined | null
const preValueObj: { bot: PreValue; script: PreValue; email: PreValue; glip: PreValue } = {
  bot: '',
  script: '',
  email: '',
  glip: ''
};

interface Message {
  id: string;
  message: string;
  isMe: boolean;
  loading?: boolean;
}

const SupportedBlock = ['BotMessage', 'Chat', 'GlipSender', 'Script'];

export const Create: FC<{ edit?: boolean }> = ({ edit }) => {
  const { id } = useParams();
  const [formJson, setFormJson] = useState({
    id: 'abc',
    description: '',
    name: 'My workflow',
    versionId: '',
    versionVisibleId: ''
  });
  const [chatList, setChatList] = useState<Message[]>([{
    id: uniqueId('r'),
    message: getDefaultMessage(edit),
    isMe: false
  }]);
  const [chatInput, setChatInput] = useState('');
  const [workflowContent, setWorkflowContent] = useState('');
  // eslint-disable-next-line no-template-curly-in-string
  const flowDefinition = useMemo<Definition | undefined>(() => DSL2Json(workflowContent), [workflowContent]);
  const previewImg = useMemo<string>(() => Json2Preview(flowDefinition), [flowDefinition]);
  const chatContentRef = useRef<HTMLUListElement>(null);
  const actions = useMemo<Array<Action & {
    stateName: string
  }>>(() => ((flowDefinition || {}).states || []).reduce<Array<Action & { stateName: string }>>((l, s) => {
    l.push(...s.actions.map(a => ({ ...a, stateName: s.name })));
    return l;
  }, []), [flowDefinition]);
  const hasEmailSender = useMemo(() => actions.some(a => a.actionType === "EmailSender"), [actions]);
  const botAddinAction = useMemo<Action & {
    stateName: string
  } | undefined>(() => actions.find(a => a.actionType === "Chat"), [actions]);
  const scriptAction = useMemo<Action & {
    stateName: string
  } | undefined>(() => actions.find(a => a.actionType === "Script"), [actions]);
  const glipSenderAction = useMemo<Action & {
    stateName: string
  } | undefined>(() => actions.find(a => a.actionType === "GlipSender"), [actions]);
  const [aibot, setAIBot] = useState('');
  const [script, setScript] = useState('');
  const [emailData, setEmailData] = useState<EmailValue>({
    to: '',
    subject: '',
    // eslint-disable-next-line no-template-curly-in-string
    content: '${localData.Desc}',
  });
  const [glipSenderData, setGlipSenderData] = useState({
    clientId: '',
    clientSecret: '',
    // eslint-disable-next-line no-template-curly-in-string
    message: ''
  });
  const [channelId, setChannelId] = useState('78394499078-4034902020');
  const location = useLocation();
  const debug = useMemo(() => {
    return (new URLSearchParams(location.search)).get('debug') === 'true';
  }, [location]);
  useEffect(() => {
    console.log(workflowContent);
    console.log(flowDefinition);
  }, [flowDefinition, workflowContent]);
  useEffect(() => {
    if (id) {
      axios.get<{
        id: string
        visibleId: string
        versionId: string
        versionVisibleId: string
        content: string
        name: string
        description: string
      }>(`/bot/workflow/${id}`).then(({ data }) => {
        setFormJson(data);
        setWorkflowContent(data.content || '');
      });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  const handleChange = (e: ChangeEvent<HTMLInputElement>) => {
    setChatInput(e.target.value);
  };

  const handleClickChip = (text: string) => () => {
    setChatInput(s => s + text);
  };
  const handleSend = async () => {
    const rId = uniqueId('r');

    setChatList([...chatList, {
      id: uniqueId('m'),
      message: chatInput,
      isMe: true
    }, {
      id: rId,
      message: '',
      isMe: false,
      loading: true
    }]);
    setChatInput('');
    try {
      const { data } = await axios.post<{ answer: string; workflowContent: string }>(`/bot/workflow/${id}/messages`, {
        "question": chatInput
      });
      setWorkflowContent(data.workflowContent);
      setChatList(list => list.map(l => {
        if (l.id === rId) {
          l.loading = false;
          l.message = data.answer;
        }
        return l;
      }));
    } catch (e) {
      setChatList(list => list.filter(l => l.id !== rId));
    }
  };

  useEffect(() => {
    if (glipSenderAction) {
      let actionValueString = glipSenderAction.message;
      let crtValueString = glipSenderData.message;
      if (actionValueString !== preValueObj.glip && crtValueString === preValueObj.glip) {
        setGlipSenderData({
          clientSecret: '',
          clientId: '',
          message: actionValueString || '',
        });
        preValueObj.glip = actionValueString || '';
      }
    } else {
      if (preValueObj.glip) {
        preValueObj.glip = '';
        setGlipSenderData({
          clientSecret: '',
          clientId: '',
          // eslint-disable-next-line no-template-curly-in-string
          message: '${localData.Desc}'
        });
      }
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [glipSenderAction]);
  useEffect(() => {
    if (scriptAction) {
      if (scriptAction.scriptSource !== preValueObj.script && script === preValueObj.script) {
        setScript(scriptAction.scriptSource || '');
        preValueObj.script = scriptAction.scriptSource || '';
      }
    } else if (preValueObj.script) {
      preValueObj.script = '';
      setScript('');
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [scriptAction]);

  useEffect(() => {
    if (botAddinAction) {
      if (botAddinAction.id !== preValueObj.bot && aibot === preValueObj.bot) {
        setAIBot(botAddinAction.id || '');
        preValueObj.bot = botAddinAction.id || '';
      }
    } else if (preValueObj.bot) {
      preValueObj.bot = '';
      setAIBot('');
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [botAddinAction]);

  useEffect(() => {
    if (chatContentRef.current) {
      chatContentRef.current.scrollTop = chatContentRef.current.scrollHeight - chatContentRef.current.offsetHeight;
    }
  }, [chatList]);

  const handleRefresh = async () => {
    await axios.delete(`/bot/workflow/${id}/messages`);
    setChatList([{
      id: uniqueId('r'),
      message: getDefaultMessage(edit),
      isMe: false
    }]);
    setChatInput('');
    setWorkflowContent('');
  };

  useEffect(() => {
    if (!hasEmailSender)
      setEmailData({
        to: '',
        subject: '',
        // eslint-disable-next-line no-template-curly-in-string
        content: '${localData.Desc}',
      });
  }, [hasEmailSender]);
  const navigate = useNavigate();
  const handleSubmit = async () => {
    if (!flowDefinition) return;
    const newJson = cloneDeep(flowDefinition);
    newJson.states.forEach(state => {
      state.actions.forEach(action => {
        if (action.actionType === 'Chat') {
          action.id = aibot;
        }
        if (action.actionType === 'Script') {
          action.scriptSource = script;
        }
        if (action.actionType === 'EmailSender') {
          action.to = emailData.to;
          action.subject = emailData.subject;
          action.content = emailData.content;
        }
        if (action.actionType === 'GlipSender') {
          // action.url = glipSenderData.url;
          action.message = glipSenderData.message || '${localData.Desc}';
        }
      });
    });
    console.log(Json2Yml(newJson));
    await axios.put(`/bot/workflow/${id}`, {
      name: formJson.name,
      content: Json2Yml(newJson)
    });
    if (!debug) {
      await axios.delete(`/bot/workflow/${id}/bind/${channelId}`);
      await axios.post(`/bot/workflow/${id}/bind`, { channelId });
    }
    message.success(`Saved! ${debug ? 'Back to List and go bind your channel!' : ''}`);
    setTimeout(() => {
      navigate(`/${debug ? '?debug=true' : ''}`);
    }, 3000);
  };

  return (
    <>
      <Grid container spacing={2}>
        <Grid item xs={12}>
          <TextField value={formJson.name} fullWidth variant="outlined" label="Workflow Name" InputProps={{
            readOnly: true,
          }}
                     placeholder="Please name your workflow" onChange={e => {
            setFormJson(json => ({
              ...json,
              name: e.target.value
            }));
          }}/>
          {/*<Divider sx={{ mt: 4, mb: -2 }}/>*/}
        </Grid>
        <Grid item xs={12}>
          <Grid container spacing={2}>
            <Grid item xs>
              <Box sx={{
                border: 1,
                height: 600,
                borderRadius: 1,
                mt: 4,
                borderColor: 'rgba(255,255,255,0.23)',
                position: 'relative'
              }}>
                <Tooltip title="Clear AI 🧠">
                  <IconButton onClick={handleRefresh} style={{ position: 'absolute', right: 10, top: 10, zIndex: 5 }}>
                    <RefreshIcon/>
                  </IconButton>
                </Tooltip>
                <div style={{ height: '100%', display: 'flex', flexDirection: 'column' }}>
                  <List ref={chatContentRef}
                        style={{ flex: '1 1 auto', overflowY: 'auto', overflowX: 'hidden' }}
                        sx={{ p: 2 }}>
                    {chatList.map(chat => <ChatItem isMe={chat.isMe} message={chat.message} key={chat.id}
                                                    loading={chat.loading}/>)}
                  </List>
                  <Box sx={{ display: 'flex', flex: '0 0 auto', p: 2, borderTop: 1, borderColor: 'divider' }}>
                    <Grid container spacing={2}>
                      {/*<Grid item xs={12} style={{ gap: 8 }}>*/}
                      {/*  <Stack direction="row" spacing={1}>*/}
                      {/*    <Typography variant="body2">*/}
                      {/*      Supported Action Types:*/}
                      {/*    </Typography>*/}
                      {/*    {SupportedBlock.map(label => (*/}
                      {/*      <Chip label={label} key={label} size="small" clickable color="primary"*/}
                      {/*            onClick={handleClickChip(label)}/>))}*/}
                      {/*  </Stack>*/}
                      {/*</Grid>*/}
                      <Grid item xs>
                        <TextField fullWidth size="small" onChange={handleChange}
                                   onKeyDown={(e: KeyboardEvent<HTMLInputElement>) => {
                                     if (e.key === 'Enter' && chatInput.length) {
                                       e.preventDefault();
                                       handleSend();
                                     }
                                   }}
                                   autoFocus
                                   variant="outlined"
                                   focused placeholder="Enter your goal ..."
                                   multiline maxRows={3}
                                   value={chatInput}/>
                      </Grid>
                      <Grid item xs="auto" justifyContent="center">
                        <Button variant="contained" size="large" disabled={!chatInput.length} onClick={handleSend}>
                          <SendIcon/>
                        </Button>
                      </Grid>
                    </Grid>
                  </Box>
                </div>
              </Box>
            </Grid>
            <Grid item xs={4}>
              <Box sx={{ border: 1, height: 600, borderRadius: 1, p: 2, mt: 4, borderColor: 'rgba(255,255,255,0.23)' }}>
                {!previewImg ? <>👋 The flow chart will be previewed here! <br/>😻 Tell the AI robot what you want to
                    do! </> :
                  <img src={previewImg} style={{ width: '100%', height: '100%' }} alt="preview"/>}
              </Box>
            </Grid>
          </Grid>
          <Divider sx={{ mt: 4, mb: 4 }}/>
          {!!scriptAction || hasEmailSender || !!botAddinAction ?
            <Typography variant="subtitle2" sx={{ mb: 2 }}>
              Please fill in the following information related to your workflow nodes. 👇🏻👇🏻</Typography> : null}
          {hasEmailSender && <EmailSender value={emailData} onChange={setEmailData}/>}
          {!!botAddinAction && <AIBot name={botAddinAction.stateName} value={aibot} onChange={setAIBot}/>}
          {!!scriptAction && <ScriptBlock name={scriptAction.stateName} value={script} onChange={setScript}/>}
          {!!glipSenderAction && <GlipSender debug={debug} value={glipSenderData} onChange={setGlipSenderData}/>}
        </Grid>
      </Grid>
      <div style={{ height: 80 }}></div>
      <Box
        sx={{
          position: 'fixed',
          bottom: 0,
          left: 0,
          right: 0,
          p: 2,
          boxShadow: 2,
          borderTop: 1,
          zIndex: 10,
          bgcolor: 'background.default',
          borderColor: 'divider',
          display: { xs: 'flex' }, justifyContent: "flex-end"
        }}>
        <Button variant="contained" style={{ width: 100 }} onClick={handleSubmit}>Save</Button>
      </Box>
    </>
  );
};
